#General Util Functions
str=(obj)->
    if obj == null then "null"
    else if typeof obj == "undefined" then "undefined"
    else obj.toString()

#General Testing Code
class Test
    constructor:(@name, @func)->
        @num = 0
    expect:(num)->
        @num = num
    equal:(arg1, arg2, message="''")->
        @num--
        if arg1 != arg2 then throw "NotEqual: '#{str(arg1)}' does not equal '#{str(arg2)}'\n   #{message}"
    deepEqual:(arg1, arg2, message="")->
        @num--
        if not require('deep-equal')(arg1, arg2) then throw "NotEqual: '#{str(arg1)}' does not equal '#{str(arg2)}'\n   #{message}"
    ok:(bool,message="")->
        @num--
        if not bool then throw "NotOk: false was passed to ok\n   #{message}"
    done:(message="")->
        if @num != 0 then throw "NotDone: #{str(@num)} more checks were expected before done was called\n   #{message}"
    run:()->
        @func.call(this)
        @done()
        
test=(name, func)->
    t = new Test(name, func)
    exports[name]=()->t.run()

exports.RunAll = (throwException)->
    for name of exports
        if name != "RunAll"
            if throwException then exports[name]()
            else
                try
                    exports[name]()
                catch ex
                    console.log "Error in Test '#{name}'"
                    console.log "Message: #{ex}"
                    console.log "Stack:\n#{ex.stack}"
                    console.log ''
    return

#File specific test functions
jsonLint=require('../src/durable-json-lint')
simpleTest=(inJson, outJson, errorCount=null)->
    test inJson + " -> " + outJson, ()->
        @num++
        res = jsonLint(inJson)
        @equal(outJson, res.json)
        if errorCount
            @num++
            @equal(errorCount, res.errors.length)

simpleTest('{"test":"test"}','{"test":"test"}')
simpleTest('{test:1}','{"test":1}')
simpleTest('{test:test}','{"test":"test"}')
simpleTest('[1,test,"test",0x11]','[1,"test","test",17]')
simpleTest('[1,[1,2,3],test,function(){}]','[1,[1,2,3],"test",null]')
simpleTest("{test:'test'}",'{"test":"test"}')
simpleTest('{"test":"test1","test":"test2"}','{"test":"test1"}',1)
simpleTest('',null,1)
simpleTest('-1','-1',0)
simpleTest('[1,]','[1]',0)
simpleTest('[1]//test','[1]',1)
simpleTest('[1]/*test*/','[1]',1)
test "crash test", ()->
    @expect(1)
    res = jsonLint("{fun:cool")
    @ok(res.errors.length==1 && !res.json?)